@@ -28,9 +28,12 @@
 #include <apt-pkg/sourcelist.h>
 #include <glib.h>
 #include <cassert>
+
+#include <gdk/gdk.h>
+
 #include "rgrepositorywin.h"
 #include "rguserdialog.h"
-#include "rgmisc.h"
+#include "rgutils.h"
 #include "config.h"
 #include "i18n.h"
 
@@ -39,6 +42,8 @@ enum { ITEM_TYPE_RPM,
    ITEM_TYPE_RPMSRC,
    ITEM_TYPE_RPMDIR,
    ITEM_TYPE_RPMSRCDIR,
+   ITEM_TYPE_REPOMD,
+   ITEM_TYPE_REPOMDSRC,
    ITEM_TYPE_DEB,
    ITEM_TYPE_DEBSRC
 };
@@ -48,7 +53,9 @@ enum { ITEM_TYPE_DEB,
    ITEM_TYPE_RPM,
    ITEM_TYPE_RPMSRC,
    ITEM_TYPE_RPMDIR,
-   ITEM_TYPE_RPMSRCDIR
+   ITEM_TYPE_RPMSRCDIR,
+   ITEM_TYPE_REPOMD,
+   ITEM_TYPE_REPOMDSRC
 };
 #endif
 
@@ -64,7 +71,12 @@ enum {
    N_SOURCES_COLUMNS
 };
 
- void RGRepositoryEditor::item_toggled(GtkCellRendererToggle *cell, 
+enum {
+   COL_NAME,
+   COL_TYPE,
+};
+
+void RGRepositoryEditor::item_toggled(GtkCellRendererToggle *cell, 
 				       gchar *path_str, gpointer data)
 {
    RGRepositoryEditor *me = (RGRepositoryEditor *)g_object_get_data(G_OBJECT(cell), "me");
@@ -100,6 +112,8 @@ enum {
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                       STATUS_COLUMN, toggle_item, -1);
 
+   me->_dirty = true;
+
    /* clean up */
    g_free(section);
    gtk_tree_path_free(path);
@@ -107,7 +121,7 @@ enum {
 
 
 RGRepositoryEditor::RGRepositoryEditor(RGWindow *parent)
-: RGGladeWindow(parent, "repositories")
+   : RGGtkBuilderWindow(parent, "repositories"), _dirty(false)
 {
    //cout << "RGRepositoryEditor::RGRepositoryEditor(RGWindow *parent)"<<endl;
    assert(_win);
@@ -117,7 +131,7 @@ RGRepositoryEditor::RGRepositoryEditor(RGWindow *parent)
 
    setTitle(_("Repositories"));
    gtk_window_set_modal(GTK_WINDOW(_win), TRUE);
-   gtk_window_set_policy(GTK_WINDOW(_win), TRUE, TRUE, FALSE);
+   //gtk_window_set_policy(GTK_WINDOW(_win), TRUE, TRUE, FALSE);
 
    // build gtktreeview
    _sourcesListStore = gtk_list_store_new(N_SOURCES_COLUMNS,
@@ -129,7 +143,7 @@ RGRepositoryEditor::RGRepositoryEditor(RGWindow *parent)
                                           G_TYPE_STRING,
                                           G_TYPE_POINTER, GDK_TYPE_COLOR);
 
-   _sourcesListView = glade_xml_get_widget(_gladeXML, "treeview_repositories");
+   _sourcesListView = GTK_WIDGET(gtk_builder_get_object(_builder, "treeview_repositories"));
    gtk_tree_view_set_model(GTK_TREE_VIEW(_sourcesListView),
                            GTK_TREE_MODEL(_sourcesListStore));
 
@@ -207,96 +221,158 @@ RGRepositoryEditor::RGRepositoryEditor(RGWindow *parent)
    g_signal_connect(G_OBJECT(select), "changed",
                     G_CALLBACK(SelectionChanged), this);
 
-   //_cbEnabled = glade_xml_get_widget(_gladeXML, "checkbutton_enabled");
+   //_cbEnabled = GTK_WIDGET(gtk_builder_get_object(_builder, "checkbutton_enabled"));
 
-   _optType = glade_xml_get_widget(_gladeXML, "optionmenu_type");
-   _optTypeMenu = gtk_menu_new();
+   _optType = GTK_WIDGET(gtk_builder_get_object(_builder, "combo_type"));
 
-   GtkWidget *item;
+   renderer = gtk_cell_renderer_text_new();
+   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (_optType), renderer, TRUE);
+   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (_optType), renderer,
+                                   "text", COL_NAME,
+                                   NULL);
+   _optTypeMenu = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
+
+   GtkTreeIter item;
 #if HAVE_RPM
-   item = gtk_menu_item_new_with_label("rpm");
-   gtk_menu_append(GTK_MENU(_optTypeMenu), item);
-   gtk_widget_show(item);
-   gtk_object_set_data(GTK_OBJECT(item), "id", (gpointer) ITEM_TYPE_RPM);
+   gtk_list_store_append(_optTypeMenu, &item);
+   gtk_list_store_set(_optTypeMenu, &item, 
+                      COL_NAME, "rpm", 
+                      COL_TYPE, ITEM_TYPE_RPM,
+                      -1);
 
-   item = gtk_menu_item_new_with_label("rpm-src");
-   gtk_menu_append(GTK_MENU(_optTypeMenu), item);
-   gtk_widget_show(item);
-   gtk_object_set_data(GTK_OBJECT(item), "id", (gpointer) ITEM_TYPE_RPMSRC);
+   gtk_list_store_append(_optTypeMenu, &item);
+   gtk_list_store_set(_optTypeMenu, &item, 
+                      COL_NAME, "rpm-src", 
+                      COL_TYPE, ITEM_TYPE_RPMSRC,
+                      -1);
 
-   item = gtk_menu_item_new_with_label("rpm-dir");
-   gtk_menu_append(GTK_MENU(_optTypeMenu), item);
-   gtk_widget_show(item);
-   gtk_object_set_data(GTK_OBJECT(item), "id", (gpointer) ITEM_TYPE_RPMDIR);
+   gtk_list_store_append(_optTypeMenu, &item);
+   gtk_list_store_set(_optTypeMenu, &item, 
+                      COL_NAME, "rpm-dir", 
+                      COL_TYPE, ITEM_TYPE_RPMDIR,
+                      -1);
 
-   item = gtk_menu_item_new_with_label("rpm-src-dir");
-   gtk_menu_append(GTK_MENU(_optTypeMenu), item);
-   gtk_widget_show(item);
-   gtk_object_set_data(GTK_OBJECT(item), "id", (gpointer) ITEM_TYPE_RPMSRCDIR);
+   gtk_list_store_append(_optTypeMenu, &item);
+   gtk_list_store_set(_optTypeMenu, &item, 
+                      COL_NAME, "rpm-src-dir",
+                      COL_TYPE, ITEM_TYPE_RPMSRCDIR,
+                      -1);
+
+   gtk_list_store_append(_optTypeMenu, &item);
+   gtk_list_store_set(_optTypeMenu, &item, 
+                      COL_NAME, "repomd",
+                      COL_TYPE, ITEM_TYPE_REPOMD,
+                      -1);
+
+   gtk_list_store_append(_optTypeMenu, &item);
+   gtk_list_store_set(_optTypeMenu, &item, 
+                      COL_NAME, "repomd-src",
+                      COL_TYPE, ITEM_TYPE_REPOMDSRC,
+                      -1);
 #else
-   item = gtk_menu_item_new_with_label(_("Binary (deb)"));
-   gtk_menu_append(GTK_MENU(_optTypeMenu), item);
-   gtk_widget_show(item);
-   gtk_object_set_data(GTK_OBJECT(item), "id", (gpointer) ITEM_TYPE_DEB);
+   gtk_list_store_append(_optTypeMenu, &item);
+   gtk_list_store_set(_optTypeMenu, &item, 
+                      COL_NAME, _("Binary (deb)"),
+                      COL_TYPE, ITEM_TYPE_DEB,
+                      -1);
 
-   item = gtk_menu_item_new_with_label(_("Source (deb-src)"));
-   gtk_menu_append(GTK_MENU(_optTypeMenu), item);
-   gtk_widget_show(item);
-   gtk_object_set_data(GTK_OBJECT(item), "id", (gpointer) ITEM_TYPE_DEBSRC);
+   gtk_list_store_append(_optTypeMenu, &item);
+   gtk_list_store_set(_optTypeMenu, &item, 
+                      COL_NAME, _("Source (deb-src)"),
+                      COL_TYPE, ITEM_TYPE_DEBSRC,
+                      -1);
 #endif
-   gtk_option_menu_set_menu(GTK_OPTION_MENU(_optType), _optTypeMenu);
+   gtk_combo_box_set_model(GTK_COMBO_BOX(_optType),
+                           GTK_TREE_MODEL(_optTypeMenu));
 
-   _optVendor = glade_xml_get_widget(_gladeXML, "optionmenu_vendor");
-   _optVendorMenu = gtk_menu_new();
-   item = gtk_menu_item_new_with_label(_("(no vendor)"));
-   gtk_menu_append(GTK_MENU(_optVendorMenu), item);
-   gtk_widget_show(item);
-   gtk_option_menu_set_menu(GTK_OPTION_MENU(_optVendor), _optVendorMenu);
-   gtk_object_set_data(GTK_OBJECT(item), "id", (gpointer) "");
+
+   // the (rpm) vendor menu
+   _optVendor = GTK_WIDGET(gtk_builder_get_object(_builder,
+                                                  "combo_vendor"));
+   renderer = gtk_cell_renderer_text_new();
+   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (_optVendor), renderer, TRUE);
+   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (_optVendor), renderer,
+                                   "text", 0,
+                                   NULL);
+   _optVendorMenu = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+
+   gtk_list_store_append(_optVendorMenu, &item);
+   gtk_list_store_set(_optVendorMenu, &item, 
+                      0, _("(no vendor)"),
+                      1, "",
+                      -1);
+
+   gtk_combo_box_set_model(GTK_COMBO_BOX(_optVendor),
+                           GTK_TREE_MODEL(_optVendorMenu));
 
 #ifndef HAVE_RPM
    // debian can't use the vendors menu, so we hide it
    gtk_widget_hide(GTK_WIDGET(_optVendor));
-   GtkWidget *vendors = glade_xml_get_widget(_gladeXML,"button_edit_vendors");
+   GtkWidget *vendors = GTK_WIDGET(gtk_builder_get_object
+                                   (_builder, "button_edit_vendors"));
    assert(vendors);
    gtk_widget_hide(GTK_WIDGET(vendors));
 #endif
 
-   _entryURI = glade_xml_get_widget(_gladeXML, "entry_uri");
+   _entryURI = GTK_WIDGET(gtk_builder_get_object(_builder, "entry_uri"));
    assert(_entryURI);
-   _entryDist = glade_xml_get_widget(_gladeXML, "entry_distribution");
+   _entryDist = GTK_WIDGET(gtk_builder_get_object(_builder,
+                                                  "entry_distribution"));
    assert(_entryDist);
-   _entrySect = glade_xml_get_widget(_gladeXML, "entry_sections");
+   _entrySect = GTK_WIDGET(gtk_builder_get_object(_builder, "entry_sections"));
    assert(_entrySect);
 
-   glade_xml_signal_connect_data(_gladeXML,
-                                 "on_button_ok_clicked",
-                                 G_CALLBACK(DoOK), this);
+   g_signal_connect(GTK_WIDGET(gtk_builder_get_object(_builder, "button_ok")),
+                    "clicked",
+                    G_CALLBACK(DoOK), this);
 
-   glade_xml_signal_connect_data(_gladeXML,
-                                 "on_button_cancel_clicked",
-                                 G_CALLBACK(DoCancel), this);
+   g_signal_connect(GTK_WIDGET(gtk_builder_get_object
+                               (_builder, "button_cancel")),
+                    "clicked",
+                    G_CALLBACK(DoCancel), this);
 
-   glade_xml_signal_connect_data(_gladeXML,
-                                 "on_button_edit_vendors_clicked",
-                                 G_CALLBACK(VendorsWindow), this);
+   g_signal_connect(GTK_WIDGET(gtk_builder_get_object
+                               (_builder, "button_edit_vendors")),
+                    "clicked",
+                    G_CALLBACK(VendorsWindow), this);
 
    /*
-      glade_xml_signal_connect_data(_gladeXML,
-      "on_button_clear_clicked",
+      g_signal_connect(GTK_WIDGET(gtk_builder_get_object
+      (_builder, "button_clear")),
+      "clicked",
       G_CALLBACK(DoClear),
       this);
     */
 
-   glade_xml_signal_connect_data(_gladeXML,
-                                 "on_button_add_clicked",
+   g_signal_connect(GTK_WIDGET(gtk_builder_get_object
+                               (_builder, "button_add")),
+                                 "clicked",
                                  G_CALLBACK(DoAdd), this);
 
-   glade_xml_signal_connect_data(_gladeXML,
-                                 "on_button_remove_clicked",
-                                 G_CALLBACK(DoRemove), this);
+   _upBut = GTK_WIDGET(gtk_builder_get_object(_builder, "button_up"));
+   assert(_upBut);
+   g_object_set_data(G_OBJECT(_upBut), "up", GINT_TO_POINTER(1));
+   g_signal_connect(_upBut,
+                    "clicked",
+                    G_CALLBACK(DoUpDown), this);
+   gtk_widget_set_sensitive(_upBut, FALSE);
 
-   _editTable = glade_xml_get_widget(_gladeXML, "table_edit");
+   _downBut = GTK_WIDGET(gtk_builder_get_object(_builder, "button_down"));
+   assert(_downBut);
+   g_object_set_data(G_OBJECT(_downBut), "down", GINT_TO_POINTER(0));
+   g_signal_connect(_downBut,
+                    "clicked",
+                    G_CALLBACK(DoUpDown), this);
+   gtk_widget_set_sensitive(_downBut, FALSE);
+   
+   _deleteBut = GTK_WIDGET(gtk_builder_get_object(_builder, "button_remove"));
+   assert(_deleteBut);
+   g_signal_connect(_deleteBut,
+                    "clicked",
+                    G_CALLBACK(DoRemove), this);
+   gtk_widget_set_sensitive(_deleteBut, FALSE);
+
+   _editTable = GTK_WIDGET(gtk_builder_get_object(_builder, "table_edit"));
    assert(_editTable);
    gtk_widget_set_sensitive(_editTable, FALSE);
 
@@ -329,9 +405,8 @@ bool RGRepositoryEditor::Run()
       return false;
    }
 
-   GdkColormap *cmap = gdk_colormap_get_system();
+   // FIXME: is this good enough?
    _gray.red = _gray.green = _gray.blue = 0xAA00;
-   gdk_color_alloc(cmap, &_gray);
 
    GtkTreeIter iter;
    for (SourcesListIter it = _lst.SourceRecords.begin();
@@ -368,21 +443,23 @@ bool RGRepositoryEditor::Run()
 
 void RGRepositoryEditor::UpdateVendorMenu()
 {
-   gtk_option_menu_remove_menu(GTK_OPTION_MENU(_optVendor));
-   _optVendorMenu = gtk_menu_new();
-   GtkWidget *item = gtk_menu_item_new_with_label(_("(no vendor)"));
-   gtk_menu_append(GTK_MENU(_optVendorMenu), item);
-   gtk_object_set_data(GTK_OBJECT(item), "id", (gpointer) "");
-   gtk_widget_show(item);
+   GtkTreeIter item;
+   gtk_list_store_clear(_optVendorMenu);
+
+   gtk_list_store_append(_optVendorMenu, &item);
+   gtk_list_store_set(_optVendorMenu, &item, 
+                      0, _("(no vendor)"),
+                      1, "",
+                      -1);
+
    for (VendorsListIter it = _lst.VendorRecords.begin();
         it != _lst.VendorRecords.end(); it++) {
-      item = gtk_menu_item_new_with_label((*it)->VendorID.c_str());
-      gtk_menu_append(GTK_MENU(_optVendorMenu), item);
-      gtk_widget_show(item);
-      gtk_object_set_data(GTK_OBJECT(item), "id",
-                          (gpointer) (*it)->VendorID.c_str());
+      gtk_list_store_append(_optVendorMenu, &item);
+      gtk_list_store_set(_optVendorMenu, &item, 
+                         0, (*it)->VendorID.c_str(),
+                         1, (*it)->VendorID.c_str(),
+                         -1);
    }
-   gtk_option_menu_set_menu(GTK_OPTION_MENU(_optVendor), _optVendorMenu);
 }
 
 int RGRepositoryEditor::VendorMenuIndex(string VendorID)
@@ -402,8 +479,8 @@ void RGRepositoryEditor::DoClear(GtkWidget *, gpointer data)
    RGRepositoryEditor *me = (RGRepositoryEditor *) data;
    //cout << "RGRepositoryEditor::DoClear(GtkWidget *, gpointer data)"<<endl;
 
-   gtk_option_menu_set_history(GTK_OPTION_MENU(me->_optType), 0);
-   gtk_option_menu_set_history(GTK_OPTION_MENU(me->_optVendor), 0);
+   gtk_combo_box_set_active(GTK_COMBO_BOX(me->_optType), 0);
+   gtk_combo_box_set_active(GTK_COMBO_BOX(me->_optVendor), 0);
    gtk_entry_set_text(GTK_ENTRY(me->_entryURI), "");
    gtk_entry_set_text(GTK_ENTRY(me->_entryDist), "");
    gtk_entry_set_text(GTK_ENTRY(me->_entrySect), "");
@@ -427,19 +504,21 @@ void RGRepositoryEditor::DoAdd(GtkWidget *, gpointer data)
                       SECTIONS_COLUMN, "",
                       RECORD_COLUMN, (gpointer) (rec),
                       DISABLED_COLOR_COLUMN, NULL, -1);
+
    GtkTreeSelection *selection;
-   selection =
-      gtk_tree_view_get_selection(GTK_TREE_VIEW(me->_sourcesListView));
+   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(me->_sourcesListView));
    gtk_tree_selection_unselect_all(selection);
    gtk_tree_selection_select_iter(selection, &iter);
+
    GtkTreePath *path;
-   path = gtk_tree_model_get_path(GTK_TREE_MODEL(me->_sourcesListStore),
-                                  &iter);
+   path = gtk_tree_model_get_path(GTK_TREE_MODEL(me->_sourcesListStore), &iter);
    gtk_tree_view_set_cursor(GTK_TREE_VIEW(me->_sourcesListView),
                             path, NULL, false);
    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(me->_sourcesListView), path,
                                 NULL, TRUE, 0.0, 0.0);
    gtk_tree_path_free(path);
+
+   me->_dirty=true;
 }
 
 void RGRepositoryEditor::doEdit()
@@ -468,8 +547,14 @@ void RGRepositoryEditor::doEdit()
    if (!status)
       rec->Type |= SourcesList::Disabled;
 
-   GtkWidget *menuitem = gtk_menu_get_active(GTK_MENU(_optTypeMenu));
-   switch ((long)(gtk_object_get_data(GTK_OBJECT(menuitem), "id"))) {
+   GtkTreeIter item;
+   int type;
+   gtk_combo_box_get_active_iter(GTK_COMBO_BOX(_optType), &item);
+   gtk_tree_model_get(GTK_TREE_MODEL(_optTypeMenu), &item,
+                      1, &type,
+                      -1);
+
+   switch (type) {
       case ITEM_TYPE_DEB:
          rec->Type |= SourcesList::Deb;
          break;
@@ -488,14 +573,24 @@ void RGRepositoryEditor::doEdit()
       case ITEM_TYPE_RPMSRCDIR:
          rec->Type |= SourcesList::RpmSrcDir;
          break;
+      case ITEM_TYPE_REPOMD:
+         rec->Type |= SourcesList::Repomd;
+         break;
+      case ITEM_TYPE_REPOMDSRC:
+         rec->Type |= SourcesList::RepomdSrc;
+         break;
       default:
          _userDialog->error(_("Unknown source type"));
          return;
    }
 
-   menuitem = gtk_menu_get_active(GTK_MENU(_optVendorMenu));
-   rec->VendorID = (char *)gtk_object_get_data(GTK_OBJECT(menuitem), "id");
-
+#if 0 // PORTME, no vendor id support right now
+   gtk_combo_box_get_active_iter(GTK_COMBO_BOX(_optVendor), &item);
+   gtk_tree_model_get(GTK_TREE_MODEL(_optVendorMenu), &item,
+                      1, &type,
+                      -1);
+   rec->VendorID = type;
+#endif
    rec->URI = gtk_entry_get_text(GTK_ENTRY(_entryURI));
    rec->Dist = gtk_entry_get_text(GTK_ENTRY(_entryDist));
 
@@ -541,22 +636,20 @@ void RGRepositoryEditor::DoRemove(GtkWidget *, gpointer data)
 
    GtkTreeSelection *selection;
    GtkTreeIter iter;
-   selection =
-      gtk_tree_view_get_selection(GTK_TREE_VIEW(me->_sourcesListView));
-   if (gtk_tree_selection_get_selected
-       (selection, (GtkTreeModel **) (&me->_sourcesListStore), &iter)) {
+   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(me->_sourcesListView));
+
+   if (gtk_tree_selection_get_selected(selection, (GtkTreeModel **) (&me->_sourcesListStore), &iter)) {
       SourcesList::SourceRecord *rec;
-      gtk_tree_model_get(GTK_TREE_MODEL(me->_sourcesListStore), &iter,
-                         RECORD_COLUMN, &rec, -1);
+      gtk_tree_model_get(GTK_TREE_MODEL(me->_sourcesListStore), &iter, RECORD_COLUMN, &rec, -1);
       assert(rec);
 
       me->_lst.RemoveSource(rec);
       gtk_list_store_remove(me->_sourcesListStore, &iter);
       if (me->_lastIter != NULL)
-         gtk_tree_iter_free(me->_lastIter);
+	gtk_tree_iter_free(me->_lastIter);
       me->_lastIter = NULL;
    }
-
+   me->_dirty=true;
 }
 
 void RGRepositoryEditor::DoOK(GtkWidget *, gpointer data)
@@ -575,12 +668,12 @@ void RGRepositoryEditor::DoOK(GtkWidget *, gpointer data)
    }
 
    gtk_main_quit();
-   me->_applied = true;
+   me->_applied = me->_dirty;
 }
 
 void RGRepositoryEditor::DoCancel(GtkWidget *, gpointer data)
 {
-   //GtkUI::SrcEditor *This = (GtkUI::SrcEditor *)data;
+   //RGRepositoryEditor *me = (RGRepositoryEditor *)data;
    gtk_main_quit();
 }
 
@@ -596,6 +689,10 @@ void RGRepositoryEditor::SelectionChanged(GtkTreeSelection *selection,
    GtkTreeModel *model;
    gtk_widget_set_sensitive(me->_editTable, TRUE);
 
+   gtk_widget_set_sensitive(me->_upBut, TRUE);
+   gtk_widget_set_sensitive(me->_downBut, TRUE);
+   gtk_widget_set_sensitive(me->_deleteBut, TRUE);
+   
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
       me->doEdit();             // save the old row
       if (me->_lastIter != NULL)
@@ -616,9 +713,13 @@ void RGRepositoryEditor::SelectionChanged(GtkTreeSelection *selection,
          id = ITEM_TYPE_RPMDIR;
       else if (rec->Type & SourcesList::RpmSrcDir)
          id = ITEM_TYPE_RPMSRCDIR;
-      gtk_option_menu_set_history(GTK_OPTION_MENU(me->_optType), id);
+      else if (rec->Type & SourcesList::Repomd)
+         id = ITEM_TYPE_REPOMD;
+      else if (rec->Type & SourcesList::RepomdSrc)
+         id = ITEM_TYPE_REPOMDSRC;
+      gtk_combo_box_set_active(GTK_COMBO_BOX(me->_optType), id);
 
-      gtk_option_menu_set_history(GTK_OPTION_MENU(me->_optVendor),
+      gtk_combo_box_set_active(GTK_COMBO_BOX(me->_optVendor),
                                   me->VendorMenuIndex(rec->VendorID));
 
       gtk_entry_set_text(GTK_ENTRY(me->_entryURI), utf8(rec->URI.c_str()));
@@ -626,24 +727,79 @@ void RGRepositoryEditor::SelectionChanged(GtkTreeSelection *selection,
       gtk_entry_set_text(GTK_ENTRY(me->_entrySect), "");
 
       for (unsigned int I = 0; I < rec->NumSections; I++) {
-         gtk_entry_append_text(GTK_ENTRY(me->_entrySect),
-                               utf8(rec->Sections[I].c_str()));
-         gtk_entry_append_text(GTK_ENTRY(me->_entrySect), " ");
+         int pos = gtk_editable_get_position(GTK_EDITABLE(me->_entrySect));
+         gtk_editable_insert_text(GTK_EDITABLE(me->_entrySect),
+                                  utf8(rec->Sections[I].c_str()),
+                                  -1,
+                                  &pos);
+         gtk_editable_insert_text(GTK_EDITABLE(me->_entrySect), " ", -1, &pos);
       }
    } else {
       //cout << "no selection" << endl;
       gtk_widget_set_sensitive(me->_editTable, FALSE);
+      
+      gtk_widget_set_sensitive(me->_upBut, FALSE);
+      gtk_widget_set_sensitive(me->_downBut, FALSE);
+      gtk_widget_set_sensitive(me->_deleteBut, FALSE);
    }
 }
 
 void RGRepositoryEditor::VendorsWindow(GtkWidget *, void *data)
 {
+#if 0 // PORTME
    RGRepositoryEditor *me = (RGRepositoryEditor *) data;
    RGVendorsEditor w(me, me->_lst);
    w.Run();
    GtkWidget *menuitem = gtk_menu_get_active(GTK_MENU(me->_optVendorMenu));
-   string VendorID = (char *)gtk_object_get_data(GTK_OBJECT(menuitem), "id");
+   string VendorID = (char *)g_object_get_data(G_OBJECT(menuitem), "id");
    me->UpdateVendorMenu();
    gtk_option_menu_set_history(GTK_OPTION_MENU(me->_optVendor),
                                me->VendorMenuIndex(VendorID));
+#endif
+}
+
+void RGRepositoryEditor::DoUpDown(GtkWidget *self, gpointer data)
+{
+   RGRepositoryEditor *me = (RGRepositoryEditor *) data;
+   //cout << "RGRepositoryEditor::DoUpDown(GtkWidget *, gpointer data)"<<endl;
+
+   GtkTreeIter iter;
+   GtkTreeIter iter2;
+
+   GtkTreeSelection *selection;
+   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(me->_sourcesListView));
+   gtk_tree_selection_get_selected (selection, (GtkTreeModel **) (&me->_sourcesListStore), &iter);
+
+   GtkTreePath *path;
+   path = gtk_tree_model_get_path(GTK_TREE_MODEL(me->_sourcesListStore), &iter);
+
+   // check if it does up or down
+   gboolean up = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(self), "up"));
+   if(up)
+      gtk_tree_path_prev( path );
+   else
+      gtk_tree_path_next( path );
+
+   // check if we can get a iter at the given path
+   if(!gtk_tree_model_get_iter(GTK_TREE_MODEL(me->_sourcesListStore), 
+			       &iter2, path) )
+      return;
+   
+   gtk_list_store_swap( me->_sourcesListStore, &iter, &iter2 );
+   gtk_tree_path_free(path);
+   
+   SourcesList::SourceRecord *rec;
+   gtk_tree_model_get(GTK_TREE_MODEL(me->_sourcesListStore), &iter,
+		      RECORD_COLUMN, &rec, -1);
+   assert(rec);
+   SourcesList::SourceRecord *rec_p;
+   gtk_tree_model_get(GTK_TREE_MODEL(me->_sourcesListStore), &iter2,
+		      RECORD_COLUMN, &rec_p, -1); 
+   assert(rec_p);
+
+   // Insert rec before rec_p
+   if(up)
+     me->_lst.SwapSources(rec_p, rec);
+   else
+     me->_lst.SwapSources(rec, rec_p);
 }
